"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const loggerhythm_1 = require("loggerhythm");
const uuid = require("uuid");
const logger = loggerhythm_1.Logger.createLogger('pprocessengine:external_task:worker');
class ExternalTaskWorker {
    constructor(externalTaskApi) {
        // eslint-disable-next-line @typescript-eslint/member-naming
        this._workerId = uuid.v4();
        this.lockDuration = 30000;
        this.externalTaskApi = undefined;
        this.externalTaskApi = externalTaskApi;
    }
    get workerId() {
        return this._workerId;
    }
    async waitForAndHandle(identity, topic, maxTasks, longpollingTimeout, handleAction) {
        const keepPolling = true;
        while (keepPolling) {
            const externalTasks = await this.fetchAndLockExternalTasks(identity, topic, maxTasks, longpollingTimeout);
            const interval = setInterval(async () => this.extendLocks(identity, externalTasks), this.lockDuration - 5000);
            const executeTaskPromises = [];
            for (const externalTask of externalTasks) {
                executeTaskPromises.push(this.executeExternalTask(identity, externalTask, handleAction));
            }
            await Promise.all(executeTaskPromises);
            clearInterval(interval);
        }
    }
    async fetchAndLockExternalTasks(identity, topic, maxTasks, longpollingTimeout) {
        try {
            return await this
                .externalTaskApi
                .fetchAndLockExternalTasks(identity, this.workerId, topic, maxTasks, longpollingTimeout, this.lockDuration);
        }
        catch (error) {
            logger.error(error);
            await this.sleep(1000);
            return this.fetchAndLockExternalTasks(identity, topic, maxTasks, longpollingTimeout);
        }
    }
    async executeExternalTask(identity, externalTask, handleAction) {
        try {
            const result = await handleAction(externalTask);
            await result.sendToExternalTaskApi(this.externalTaskApi, identity, this.workerId);
        }
        catch (error) {
            logger.error(error);
            await this.externalTaskApi.handleServiceError(identity, this.workerId, externalTask.id, error.message, '');
        }
    }
    async extendLocks(identity, externalTasks) {
        for (const externalTask of externalTasks) {
            await this.externalTaskApi.extendLock(identity, this.workerId, externalTask.id, this.lockDuration);
        }
    }
    async sleep(milliseconds) {
        return new Promise((resolve) => {
            setTimeout(() => { resolve(); }, milliseconds);
        });
    }
}
exports.ExternalTaskWorker = ExternalTaskWorker;
//# sourceMappingURL=external_task_worker.js.map